﻿using UnityEngine;
using System;
using System.Collections;

public class GameState : MonoBehaviour 
{
	public Transform playerTransform;
	public Rigidbody playerRigidBody;
	public Transform playerShell;
	public float currPlayerSpeed = 0.0f;
	public float requestedPlayerSpeed = 0.0f;
	public float playerSpeedAcceleration= 0.1f;
	public float playerMaxSpeed = 5.0f;

	public Transform playerFanAnchor;
	public RotateYAxis playerRotateYAxisComponent;
	private float currentYawRatePerSec = 0.0f;
	private float requestedYawRatePerSec = 360.0f;
	public float yawRateAcceleration = 45.0f;
	private float maxYawRatePerSec = 1040.0f;
	public float playerShellRotateRate = 1.0f;

	public float moveDeadZone = 0.2f;
	private bool prevHorzMove = false;
	private bool prevVertMove = false;

	public float currentVerticalPlayerSpeed = 0.0f;
	public float requestedVerticalPlayerSpeed = 0.0f;
	public float verticalPlayerSpeedAcceleration = 25.0f;
	private float maxVerticalPlayerSpeed = 2500.0f;
	public float playerUpdraftSpeedPerSec = 250.0f;
	public float playerDescentSpeedPerSec = 50.0f;
	public bool playerInUpdraft = false;

	public float upDraftYawRate = 720.0f;
	public float downDraftYawRate = 180.0f;

	public Transform healthBarBug;
	private float maxPlayerHeight = 25.0f;
	private float yHealthBarBugDim;
	private float yHealthBarBugOrigPosition;
	private float yHealthBarBugDims = 115.0f;

	public GameObject clickTargetSymbol;

	public GameObject gameOverText;

	public bool reportedTerrainCollision = false;
	public ContactPoint reportedTerrainCollisionContactPoint;

	public float enemySpawnTimeCounter = 0.0f;
	public float enemySpawnTimeInterval = 5.0f;
	private GameObject[] listOfEnemySpawns;
	private GameObject[] listOfEnemyNavPoints;

	public int maxEnemyAlive = 16;
	public ArrayList listOfSpawnedEnemies = new ArrayList();

	public int randomSeed = 1337;

	public GameObject explosionParticlSystem;
	public GameObject largeExplosionParticlSystem;

	public GameObject[] enemySpawnables;

	public GameObject timeText;
	public float ingameTime = 0.0f;
	public float hiscoreIngameTime = 0.0f;

	public bool gameOverFlag = false;
	public bool waitingForEndOfGame = false;

	private float taskTextDisplayTime = 0.0f;
	public GameObject taskText;
	
	public AudioClip fireSfx;
	public AudioClip hitTargetSfx;
	public AudioClip hitTerrainSfx;
	public AudioClip explosionSfx;
	public AudioClip gameOverSfx;
	public AudioClip moveSfx;
	public AudioClip updraftSfx;
	public AudioClip toCrystalsSfx;

	public float lastFireTimeSecs = 0.0f;
	public float fireIntervalSecs = 1.0f;

	public float enemyMovementSpeed = 1.5f;
	public float enemyMovementSpeedIncrement = 0.1f;

	void Awake()
	{
		hiscoreIngameTime = PlayerPrefs.GetFloat("hiscoreIngameTime");
	}

	void OnDestroy()
	{
		if(ingameTime > hiscoreIngameTime)
		{
			PlayerPrefs.SetFloat("hiscoreIngameTime", ingameTime);
		}
	}

	void Start () 
	{
		UnityEngine.Random.seed = randomSeed;

		yHealthBarBugOrigPosition = healthBarBug.transform.position.y;
		
		listOfEnemySpawns = GameObject.FindGameObjectsWithTag("EnemySpawnPoint");
		listOfEnemyNavPoints = GameObject.FindGameObjectsWithTag("EnemyNavPoint");

		taskText.SetActive (true);
	}
	
	void Update () 
	{
		if(gameOverFlag)
		{
			if(!waitingForEndOfGame)
			{
				gameObject.GetComponent<AudioSource>().PlayOneShot(gameOverSfx, 1.0f);

				HandleAllEnemyStopping();

				StartCoroutine("HandleGameOverDisplay");
				waitingForEndOfGame = true;
			}

			return;
		}

		if(reportedTerrainCollision)
		{
			HandleAllEnemyStopping();

			Instantiate(largeExplosionParticlSystem, reportedTerrainCollisionContactPoint.point,  Quaternion.identity);
			gameObject.GetComponent<AudioSource>().PlayOneShot(explosionSfx, 1.0f);

			gameOverFlag = true;

			reportedTerrainCollision = false;

			return;
		}

		ResetRequestedPlayerVerticalSpeed();
		if(playerInUpdraft)
		{
			requestedYawRatePerSec = upDraftYawRate;
			AddRequestedPlayerVerticalSpeed(playerUpdraftSpeedPerSec);

			gameObject.GetComponent<AudioSource>().PlayOneShot(gameOverSfx, 0.1f);
		}
		else
		{
			requestedYawRatePerSec = downDraftYawRate;
			AddRequestedPlayerVerticalSpeed(-playerDescentSpeedPerSec);
		}

		HandleEnemyMoving();
		HandleEnemySpawning();

		HandleMouseTargetting();
		HandlePlayerInput();
		HandlePlayerFanRotation();

		HandleHealthBarBug();
		HandleIngameTime();

		if(taskText.activeSelf) 
		{
			taskTextDisplayTime += Time.deltaTime;
			if(taskTextDisplayTime >= 10.0f)
			{
				taskText.SetActive (false);
			}
		}
	}

	void HandleIngameTime ()
	{
		int hours = (int)(ingameTime / (60.0f * 60.0f));
		int minutes = (int)((ingameTime / 60.0f) % 60.0f);
		int seconds = (int)(ingameTime % 60.0f);

		string timeDisplayText = String.Format("{0}:{1:00}:{2:00}", hours, minutes, seconds);
		timeText.GetComponent<UnityEngine.UI.Text>().text = timeDisplayText;
		
		ingameTime += Time.deltaTime;
	}
	
	IEnumerator HandleGameOverDisplay() 
	{
		gameOverText.SetActive (true);
		
		yield return new WaitForSeconds(5);
		
		Application.LoadLevel("Titlescreen");
	}

	void HandleEnemySpawning () 
	{
		if(listOfSpawnedEnemies.Count >= maxEnemyAlive)
		{
			return;
		}

		enemySpawnTimeCounter += Time.deltaTime;

		while(enemySpawnTimeCounter > enemySpawnTimeInterval)
		{
			int randomIndex = (UnityEngine.Random.Range(0, listOfEnemySpawns.Length)); 
			Transform currentSpawnSelectedTransform = listOfEnemySpawns[randomIndex].transform;

			Vector3 newPos = new Vector3(currentSpawnSelectedTransform.position.x, currentSpawnSelectedTransform.position.y + 0.01f, currentSpawnSelectedTransform.position.z);

			int randomEnemyIndex = (UnityEngine.Random.Range(0, enemySpawnables.Length)); 

			GameObject newEnemy = Instantiate(enemySpawnables[randomEnemyIndex], newPos, currentSpawnSelectedTransform.rotation) as GameObject;
			newEnemy.GetComponent<HandleEnemy>().enemyID = listOfSpawnedEnemies.Count;

			int randomNavPointIndex = (UnityEngine.Random.Range(0, listOfEnemyNavPoints.Length)); 

			newEnemy.GetComponent<NavMeshAgent>().destination = listOfEnemyNavPoints[randomNavPointIndex].transform.position;
			newEnemy.GetComponent<NavMeshAgent>().speed = enemyMovementSpeed;

			listOfSpawnedEnemies.Add(newEnemy);

			enemySpawnTimeCounter -= enemySpawnTimeInterval;

			enemyMovementSpeed += enemyMovementSpeedIncrement;
		}
	}

	void HandleEnemyMoving() 
	{
		for(int index = 0; index < listOfSpawnedEnemies.Count; index++)
		{
			GameObject currentEnemy = listOfSpawnedEnemies[index] as GameObject;

			NavMeshAgent agent = currentEnemy.GetComponent<NavMeshAgent>();
			if(agent.hasPath && agent.remainingDistance <= 1.0f) 
			{
				if(currentEnemy.GetComponent<HandleEnemy>().movingToNavPoint)
				{
					currentEnemy.GetComponent<HandleEnemy>().movingToNavPoint = false;
					currentEnemy.GetComponent<NavMeshAgent>().destination = GameObject.Find ("CrystalsBunch").transform.position;

					gameObject.GetComponent<AudioSource>().PlayOneShot(toCrystalsSfx, 0.2f);
				}
				else
				{
					//listOfSpawnedEnemies.RemoveAt(index);
					//Destroy(currentEnemy);

					gameOverFlag = true;
				}
			}
		}
	}

	void HandleAllEnemyStopping ()
	{
		for(int index = 0; index < listOfSpawnedEnemies.Count; index++)
		{
			GameObject currentEnemy = listOfSpawnedEnemies[index] as GameObject;
			
			NavMeshAgent agent = currentEnemy.GetComponent<NavMeshAgent>();
			agent.Stop();
		}
	}

	void HandlePlayerInput () 
	{
		bool didControlThisFrame = false;

		float horzInput = Input.GetAxis("Horizontal");
		float vertInput = Input.GetAxis("Vertical");

		if(prevHorzMove || prevVertMove)
		{
			if(horzInput <= -moveDeadZone || horzInput >= moveDeadZone || vertInput <= -moveDeadZone || vertInput >= moveDeadZone)
			{
				didControlThisFrame = true;

				gameObject.GetComponent<AudioSource>().PlayOneShot(moveSfx, 0.02f);

				requestedPlayerSpeed += playerSpeedAcceleration;
				
			 	if(requestedPlayerSpeed > playerMaxSpeed)
				{
					requestedPlayerSpeed = playerMaxSpeed;
				}
			}
			else
			{
				requestedPlayerSpeed -= playerSpeedAcceleration;
				
				if(requestedPlayerSpeed < 0.0f)
				{
					requestedPlayerSpeed = 0.0f;
				}
			}
		}
		else 
		{
			requestedPlayerSpeed -= playerSpeedAcceleration;
			
			if(requestedPlayerSpeed < 0.0f)
			{
				requestedPlayerSpeed = 0.0f;
			}
		}
			
		if(requestedPlayerSpeed > currPlayerSpeed)
		{
			currPlayerSpeed += playerSpeedAcceleration;
			
			if(currPlayerSpeed > playerMaxSpeed)
			{
				currPlayerSpeed = playerMaxSpeed;
			}
		}
		else if(requestedPlayerSpeed < currPlayerSpeed)
		{
			currPlayerSpeed -= playerSpeedAcceleration;
			
			if(currPlayerSpeed < 0.0f)
			{
				currPlayerSpeed = 0.0f;
			}
		}

		if(requestedVerticalPlayerSpeed > currentVerticalPlayerSpeed)
		{
			currentVerticalPlayerSpeed += verticalPlayerSpeedAcceleration;
			
			if(currentVerticalPlayerSpeed > maxVerticalPlayerSpeed)
			{
				currentVerticalPlayerSpeed = maxVerticalPlayerSpeed;
			}
		}
		else if(requestedVerticalPlayerSpeed < currentVerticalPlayerSpeed)
		{
			currentVerticalPlayerSpeed -= verticalPlayerSpeedAcceleration;
			
			if(currentVerticalPlayerSpeed < -maxVerticalPlayerSpeed)
			{
				currentVerticalPlayerSpeed = -maxVerticalPlayerSpeed;
			}
		}

		Vector3 rightVector = Vector3.right * (horzInput * currPlayerSpeed);
		Vector3 forwardVector = Vector3.forward * (vertInput * currPlayerSpeed);
		Vector3 targetVector = rightVector + forwardVector;
		playerRigidBody.AddForce (targetVector * Time.deltaTime);

		playerRigidBody.AddForce (Vector3.up * (currentVerticalPlayerSpeed * Time.deltaTime));

		if(didControlThisFrame)
		{
			float rotateStep = 15.0f;
			Vector3 newRotationDir = Vector3.RotateTowards(playerTransform.forward, targetVector, rotateStep, 0.0F);
			playerShell.rotation = Quaternion.LookRotation(newRotationDir);
		}


		if(horzInput <= -moveDeadZone || horzInput >= moveDeadZone)
		{
			prevHorzMove = true;
		}
		else
		{
			prevHorzMove = false;
		}
		
		if(vertInput <= -moveDeadZone || vertInput >= moveDeadZone)
		{
			prevVertMove = true;
		}
		else
		{
			prevVertMove = false;
		}
	}

	void HandlePlayerFanRotation ()
	{
		if(requestedYawRatePerSec < 0.0f)
		{
			requestedYawRatePerSec = 0.0f;
		}
		else if(requestedYawRatePerSec > maxYawRatePerSec)
		{
			requestedYawRatePerSec = maxYawRatePerSec;
		}
		
		if(currentYawRatePerSec < requestedYawRatePerSec)
		{
			currentYawRatePerSec += yawRateAcceleration * Time.deltaTime;
		}
		else if(currentYawRatePerSec > requestedYawRatePerSec)
		{
			currentYawRatePerSec -= yawRateAcceleration * Time.deltaTime;
		}
		
		playerRotateYAxisComponent.yRatePerSec = currentYawRatePerSec;
	}

	void HandleHealthBarBug ()
	{
		float yFactor = playerTransform.position.y/ maxPlayerHeight;
		if(yFactor < 0.0f)
		{
			yFactor = 0.0f;
		}
		else if(yFactor > 1.0f)
		{
			yFactor = 1.0f;
		}

		Vector3 bugPos = healthBarBug.transform.position;
		bugPos.y = yHealthBarBugOrigPosition - ((1.0f - yFactor) * yHealthBarBugDims);
		healthBarBug.transform.position = bugPos;
	}

	void HandleMouseTargetting ()
	{
		lastFireTimeSecs += Time.deltaTime;
		if(lastFireTimeSecs < fireIntervalSecs)
		{
			return;
		}

		if (Input.GetButtonDown("Fire1")) 
		{
			lastFireTimeSecs = 0.0f;

			gameObject.GetComponent<AudioSource>().PlayOneShot(fireSfx, 1.0f);

			RaycastHit hitInfo;
			Ray ray = Camera.main.ScreenPointToRay(new Vector3(Input.mousePosition.x, Input.mousePosition.y, 0.0f));
			//int layerMask = 1 << 10;

			if (Physics.Raycast(ray, out hitInfo/*, layerMask*/))
			{
				if(hitInfo.transform.gameObject.tag == "EnemyTypeA")
				{
					for(int index = 0; index < listOfSpawnedEnemies.Count; index++)
					{
						GameObject currentEnemy = listOfSpawnedEnemies[index] as GameObject;

						if(hitInfo.transform.gameObject.GetComponent<HandleEnemy>().enemyID == currentEnemy.GetComponent<HandleEnemy>().enemyID)
						{
							listOfSpawnedEnemies.RemoveAt(index);
							Destroy(currentEnemy);

							Instantiate(largeExplosionParticlSystem, hitInfo.point,  Quaternion.identity);

							gameObject.GetComponent<AudioSource>().PlayOneShot(explosionSfx, 1.0f);
						}
					}
				}
				else
				{
					Instantiate(explosionParticlSystem, hitInfo.point,  Quaternion.identity);
					
					gameObject.GetComponent<AudioSource>().PlayOneShot(explosionSfx, 0.5f);
				}
			}
		}
	}

	public void ResetRequestedPlayerVerticalSpeed ()
	{
		requestedVerticalPlayerSpeed = 0.0f;
	}

	public void SetRequestedPlayerVerticalSpeed (float value)
	{
		requestedVerticalPlayerSpeed = value;

		if(requestedVerticalPlayerSpeed < -maxVerticalPlayerSpeed)
		{
			requestedVerticalPlayerSpeed = -maxVerticalPlayerSpeed;
		}
		else if(requestedVerticalPlayerSpeed > maxVerticalPlayerSpeed)
		{
			requestedVerticalPlayerSpeed = maxVerticalPlayerSpeed;
		}
	}
	
	public void AddRequestedPlayerVerticalSpeed (float value)
	{
		requestedVerticalPlayerSpeed += value;
		
		if(requestedVerticalPlayerSpeed < -maxVerticalPlayerSpeed)
		{
			requestedVerticalPlayerSpeed = -maxVerticalPlayerSpeed;
		}
		else if(requestedVerticalPlayerSpeed > maxVerticalPlayerSpeed)
		{
			requestedVerticalPlayerSpeed = maxVerticalPlayerSpeed;
		}
	}
}
